@@ -159,22 +159,24 @@ def write(c: pyodbc.Cursor, table_name: str, data: list[dict[str, Any]],
         in_db = {tuple(r[k] for k in primary_key) for r in query(c, sql)}
 
     if update and (update_keys := data.keys() & in_db):
-        update_data = [data[k] for k in update_keys]
+        update_data = [
+            (tuple(data[k][col] for col in columns) + tuple(data[k][col] for col in primary_key)) for k in update_keys
+        ]
 
-        # Use keyword placeholders, so we can pass a record to executemany
-        pk_cols = ' AND '.join(f'{col}=:{col}' for col in primary_key)
-        assignment = ', '.join(f'{col}=:{col}' for col in columns)
+        # Cannot use keyword placeholders because pyodbc doesn't support named paramstyle. Would be better.
+        assignment = ', '.join(f'{col}=?' for col in columns)
+        pk_cols = ' AND '.join(f'{col}=?' for col in primary_key)
         sql = f'update {table_name} set {assignment} where {pk_cols}'
 
         c.executemany(sql, update_data)
 
     if insert and (insert_keys := data.keys() - in_db):
-        insert_data = [data[k] for k in insert_keys]
+        insert_data = [tuple(data[k][col] for col in columns) for k in insert_keys]
 
         # Use keyword placeholders, so we can pass a record to executemany
-        placeholders = ', '.join(f':{name}' for name in columns)
-        columns = ', '.join(columns)
-        sql = f'insert into {table_name}({columns}) VALUES ({placeholders})'
+        placeholders = ', '.join(f'?' for name in columns)
+        text_columns = ', '.join(columns)
+        sql = f'insert into {table_name}({text_columns}) VALUES ({placeholders})'
 
         c.executemany(sql, insert_data)
 
